(*
 Representation of the internal syntax of lambda cube
 and further things needed by all other files
 
 author: Christian Mueller
*)

local
    open Support
in

type location = int

datatype mode = S | F | Fw | CC | R | RG | U | UD

val typingmode = ref S

val subtyping  = ref false

datatype Exp =
    Star of info
  | Box of info
  | Var of info * string * int
  | App of info * Exp * Exp
  | Abstr of info * string * Exp * Exp
  | Pi  of info * string * Exp * Exp
  (* additional stuff for integers and boolean *)
  | Int of info * int
  | Plus of info * Exp * Exp
  | Minus of info * Exp * Exp
  | Mult of info * Exp * Exp
  | IntNeg of info * Exp
  | IntType of info
  | BoolType of info
  | IntEqual of info * Exp * Exp
  | IntNotEqual of info * Exp * Exp
  | Less of info * Exp * Exp
  | LessEq of info * Exp * Exp
  | Greater of info * Exp * Exp
  | GreaterEq of info * Exp * Exp
  | True of info
  | False of info
  | BoolNeg of info * Exp
  | If of info * Exp * Exp * Exp       
  (* unit *)
  | UnitType of info
  | Unit of info
  (* extensions for variants *)
  | VariantSort of info * (string * Exp) list
  | Variant of info * string * Exp * Exp
  | Case of info * Exp * (string * string * Exp) list
  (* extensions for records *)
  | RecordSort of info * (string * Exp) list
  | Record of info * (string * Exp) list
  | Proj of info * Exp * string
  (* extensions for iso-recursive types *)
  | RecursiveSort of info * string * Exp
  | Fold of info * Exp * Exp
  | Unfold of info * Exp * Exp
  (* ascription *)
  | Ascribe of info * Exp * Exp
  (* built-in fixpoint operator *)
  | Fix of info * Exp
  (* references *)
  | Ref of info * Exp 
  | Loc of info * location
  | Deref of info * Exp
  | Assign of info * Exp * Exp
  | RefType of info * Exp
  (* let expression *)
  | Let of info * string * Exp * Exp
    
type do_eval = bool
datatype pp_mode = Opaque | OpaqueWithSync | Transparent
datatype pp_output = Term | Type | All

datatype Setting =
    ModeChange of pp_mode
  | OutputChange of pp_output
  | TypeModeChange of mode
  | SubtypingChange of bool

datatype Command =
    Define of string * Exp
  | ReduceDefine of string * Exp
  | ReduceStep of string * Exp
  | Protocol of Exp
  | Equiv of Exp * Exp
  | Switch of Setting
  | UseFile of string

end