(*-*-sml-*-*)

(* --------------------------------------------------------------
 * This file contains some test cases for the interpreter
 * in "simply typed" mode.
 *
 * author: Christian Mueller
 * -------------------------------------------------------------*)     

switch tm s

(* 1. some very basic declarations *)

def T = Int->Bool

rdef x = 3**2+1
rdef f = \y:Int. y << x
rdef tester = \g:T.\x:Int. if g x then 1 else ~1
rdef result_1 = tester f 3

(* 2. some examples for records *) 

rdef record = {i=3, id=\x:Int.x, b=true}
rdef result_2 = if record@b then record@id record@i else record@id 1

(* 3. examples for variants *)

def VT = <i:Int,f:Int->Int,b:Bool>

rdef variant1 = <b=true> as VT
rdef variant2 = <b=true> as VT

rdef result_31 = case variant1 of 
		    <i=x> => 1
		  | <f=x> => 2
		  | <b=x> => 3

rdef result_32 = case variant2 of
		    <i=x> => false
		  | <f=x> => false
		  | <b=x> => x

			     
(* 4.1 Fixpoint operator with iso-recursive types *)

rdef omega = \f:(Int->Int)->(Int->Int).(\x:mu X.X->(Int->Int).\y:Int. 
	            f ((unfold (mu X.X->(Int->Int)) x) x) y) 

rdef Fix = \f:(Int->Int)->(Int->Int). 
                 (omega f) (fold (mu X.X->(Int->Int)) (omega f))

rdef sum = Fix (\f:Int->Int.\x:Int.if x<=0 then 0 else x + f(x-1))

rdef result_41 = sum 4


(* 4.2 hungry function (cf. TAPL p.270) with iso-recursive types *)

def Hungry = mu A. Int -> A

rdef omega = \f:(Int->Hungry)->(Int->Hungry). (\x:mu X.X->(Int->Hungry).\y:Int.
                     f ((unfold (mu X.X->(Int->Hungry)) x) x) y) 

rdef Fix = \f:(Int->Hungry)->(Int->Hungry). 
                    (omega f) (fold (mu X.X->(Int->Hungry)) (omega f))

rdef f = Fix (\f:Int->Hungry.\n:Int. fold Hungry f)
rdef result_42 = (unfold Hungry (f 0)) 1


(* 4.3 combination of recursive types and variants *)

def Nat = mu X.<zero:Unit, succ:X>

rdef zero = fold Nat <zero=unit> as <zero:Unit, succ:Nat>
rdef succ = \n:Nat. fold Nat <succ=n> as <zero:Unit, succ:Nat>

rdef omega = \f:(Nat->Int)->(Nat->Int).(\x:mu X.X->(Nat->Int).\y:Nat. 
                   f ((unfold (mu X.X->(Nat->Int)) x) x) y) 

rdef Fix = \f:(Nat->Int)->(Nat->Int). 
                  (omega f) (fold (mu X.X->(Nat->Int)) (omega f))

rdef nat2int = Fix (\f:Nat->Int.\n:Nat. case unfold Nat n of 
					    <zero=x> => 0 
					  | <succ=x> => 1 + f x)

rdef result_43 = nat2int (succ (succ (succ zero)))


(* 
   5.1 You should now be able to define a fixpoint operator for your own
       examples. However there is a built-in fixpoint operator that makes
       live much easier as it can be used polymorphic even in S.
       Here are some short examples:
*)

(* Be aware that sum2 is tail-recursive and therefore much faster than sum *)

rdef sum_acc = fix (\f:Int->Int->Int.\acc:Int.\x:Int.
		       if x<=0 then acc else f (x+acc) (x-1))

rdef sum2 = sum_acc 0

rdef result_51 = sum2 1000

(* 
   5.2 Example from TAPL, p.144 
   This example might offer a first idea how object oriented programming 
   - especially the self construct - can be encoded. 
*)

def ff = \ieio:{iseven:Int->Bool, isodd:Int->Bool}.
	       {iseven = \x:Int. if x=0 then true else ieio@isodd (x-1),
		isodd  = \x:Int. if x=0 then false else ieio@iseven (x-1)}

def r = fix ff

def iseven = r@iseven
def isodd  = r@isodd

rdef result_521 = iseven 24
rdef result_522 = iseven 31

rdef result_523 = isodd 33
rdef result_524 = isodd 44

(* 6. let expressions for terms are also available in s,f,fw and cc *)

rdef let_example = let inc = \x:Int.x+1 in 
		      let x = 3+2 in
			  inc x
